

const inputType = {
    MOTHER_NAME : 0,
    FATHER_NAME : 1,
    BIRTHDAY : 2,
    OLDER_SIBLINGS : 3,
    MOTHER_FIRST_NAME : 4,
    FATHER_FIRST_NAME : 5,

    INVALID : -1
};



var inputComponent = Vue.component('input-component', {
    props: ["input", "german"],
    methods: {
        getTitleText : function(type) {
            switch(type) {
                case inputType.MOTHER_FIRST_NAME:
                    return this.german ? "Erster Vorname der Mutter:" :  "First Name of Mother:";
                case inputType.MOTHER_NAME:
                    return this.german ? "Geburtsname der Mutter:" :  "Maiden Name of Mother:";
                case inputType.FATHER_FIRST_NAME:
                    return this.german ? "Erster Vorname des Vaters:" :  "First Name of Father:";
                case inputType.FATHER_NAME:
                    return this.german ? "Geburtsname des Vaters:" :  "Maiden Name of Father:";
                case inputType.BIRTHDAY:
                    return this.german ? "Geburtstag (Nur der Tag):" : "Birthday (only Day):";
                case inputType.OLDER_SIBLINGS:
                    return this.german ? "Anzahl älterer Geschwister:" : "Number of older Siblings:"
                default:
                    return "NONE";
            }
        },

        getPlaceHolderText : function(type) {
            switch(type) {
                case inputType.MOTHER_FIRST_NAME:
                    return this.german ? "Vorname der Mutter" :  "Mother's Firstname";
                case inputType.MOTHER_NAME:
                    return this.german ? "Nachname der Mutter" :  "Mother's Maiden Name";
                case inputType.FATHER_FIRST_NAME:
                    return this.german ? "Vorname des Vaters" :  "Father's Firstname";
                case inputType.FATHER_NAME:
                    return this.german ? "Nachname des Vaters" :  "Father's Maiden Name";
                case inputType.BIRTHDAY:
                    return "01";
                case inputType.OLDER_SIBLINGS:
                    return "0"
                default:
                    return "NONE";
            }
        },

        getGridAreaName : function(type, inp) {
            let areaName = "";
            switch(type) {
                case inputType.MOTHER_FIRST_NAME:
                    areaName = "motherFirst";
                    break;
                case inputType.MOTHER_NAME:
                    areaName = "mother";
                    break;
                case inputType.FATHER_FIRST_NAME:
                    areaName = "fatherFirst"
                    break;
                case inputType.FATHER_NAME:
                    areaName = "father"
                    break;
                case inputType.BIRTHDAY:
                    areaName = "birthday";
                    break;
                case inputType.OLDER_SIBLINGS:
                    areaName = "siblings";
                    break;
                default:
                    return "NONE";
            }
            return areaName + (inp ? "Inp" : "Text");
        },

        getInputFieldLenght : function(type) {
            switch(type) {
                case inputType.MOTHER_FIRST_NAME:
                    return 100;
                case inputType.MOTHER_NAME:
                    return 100;
                case inputType.FATHER_FIRST_NAME:
                    return 100;
                case inputType.FATHER_NAME:
                    return 100;
                case inputType.BIRTHDAY:
                    return 2;
                case inputType.OLDER_SIBLINGS:
                    return 1;
                default:
                    return 100;
            }
        }
        
    },
    template: `<div class = "input" v-bind:id="getGridAreaName(input.type, false)">
                    <p id="gridText">{{getTitleText(input.type)}}</p>
                    <input id="gridInp" type="text" v-bind:maxlength="getInputFieldLenght(input.type)" v-bind:placeholder="getPlaceHolderText(input.type)" v-model="input.value">
                </div>`
});


var vue = new Vue({
    el: "#vue",

    data: {
        german : true,

        inputs: [
            {

                placeholder: "Mother's first name",
                value: "",
                type: inputType.MOTHER_FIRST_NAME
            },
            {
                title: "Maiden Name of Mother: ",
                placeholder: "Mother's Name",
                value: "",
                type: inputType.MOTHER_NAME
            },
            {
                title: "First Name of Father: ",
                placeholder: "Father's first Name",
                value: "",
                type: inputType.FATHER_FIRST_NAME
            },
            {
                title: "Maiden Name of Father: ",
                placeholder: "Father's Name",
                value: "",
                type: inputType.FATHER_NAME
            },
            {
                title: "Birthday (only Day): ",
                placeholder: "01",
                value: "",
                type: inputType.BIRTHDAY
            },
            {
                title: "Amount of older siblings: ",
                placeholder: "0",
                value: "",
                type: inputType.OLDER_SIBLINGS
            }
            
        ],
    },
    methods: {
        

        getCode: function() {
            this.clearErrorList();
            //console.log("Update!");
            let fatalError = false;

            let motherInp = this.getInputValue(inputType.MOTHER_NAME);
            let motherBlock = "**";
            let motherSecondChar = "*";
            
            let nameCharAmount = "**";

            if (motherInp === null) {
                this.addErrorFieldNotConnected(inputType.MOTHER_NAME);
            } else {
                if(typeof motherInp !== 'string') {
                    this.addErrorHasToBeAWord(inputType.MOTHER_NAME);
                } else 
                motherInp = motherInp.trim();
                if(/[^a-zA-Z]/.test(motherInp)) {
                    this.addErrorMustOnlyContainCharacters(inputType.MOTHER_NAME);
                    fatalError = true;
                    
                } else if(motherInp === "") {
                    this.addErrorIsEmpty(inputType.MOTHER_NAME);
                }else if (motherInp.length == 1) {
                    this.addErrorIsTooShort(inputType.MOTHER_NAME);
                } else {
                    motherBlock = motherInp[0] + motherInp[motherInp.length - 1];
                    motherSecondChar = motherInp[1];
                    nameCharAmount = motherInp.length;
                }
            }

            let fatherInp = this.getInputValue(inputType.FATHER_NAME);
            let fatherBlock = "**";
            let fatherLastSecondChar = "*";
            if (fatherInp === null) {
                this.addErrorFieldNotConnected(inputType.FATHER_NAME);
            } else {
                if(typeof fatherInp !== 'string') {
                    this.addErrorHasToBeAWord(inputType.FATHER_NAME);
                } else 
                fatherInp = fatherInp.trim();
                if(/[^a-zA-Z]/.test(fatherInp)) {
                    this.addErrorMustOnlyContainCharacters(inputType.FATHER_NAME);
                    fatalError = true;
                } else if(fatherInp === "") {
                    this.addErrorIsEmpty(inputType.FATHER_NAME);
                }else if (fatherInp.length == 1) {
                    this.addErrorIsTooShort(inputType.FATHER_NAME);
                } else {
                    
                    fatherLastSecondChar = fatherInp[fatherInp.length - 2];
                }
            }
            
            let motherFirstInp = this.getInputValue(inputType.MOTHER_FIRST_NAME);

            if (motherFirstInp === null) {
                this.addErrorFieldNotConnected(inputType.MOTHER_FIRST_NAME);
            } else {
                if(typeof motherFirstInp !== 'string') {
                    this.addErrorHasToBeAWord(inputType.MOTHER_FIRST_NAME);
                } else 
                motherFirstInp = motherFirstInp.trim();
                if(/[^a-zA-Z]/.test(motherFirstInp)) {
                    this.addErrorMustOnlyContainCharacters(inputType.MOTHER_FIRST_NAME);
                    fatalError = true;
                } else if(motherFirstInp === "") {
                    this.addErrorIsEmpty(inputType.MOTHER_FIRST_NAME);
                }else if (motherFirstInp.length == 1) {
                    this.addErrorIsTooShort(inputType.MOTHER_FIRST_NAME);
                } else {
                    nameCharAmount = motherFirstInp.length;
                }
            }

            let fatherFirstInp = this.getInputValue(inputType.FATHER_FIRST_NAME);
            console.log(fatherFirstInp);
            if (fatherFirstInp === null) {
                this.addErrorFieldNotConnected(inputType.FATHER_FIRST_NAME);
                nameCharAmount = "**";
            } else {
                if(typeof fatherFirstInp !== 'string') {
                    this.addErrorHasToBeAWord(inputType.FATHER_FIRST_NAME);
                    nameCharAmount = "**";
                } else 
                fatherFirstInp = fatherFirstInp.trim();
                if(/[^a-zA-Z]/.test(fatherFirstInp)) {
                    this.addErrorMustOnlyContainCharacters(inputType.FATHER_FIRST_NAME);
                    nameCharAmount = "**";
                    fatalError = true;
                } else if(fatherFirstInp === "") {
                    this.addErrorIsEmpty(inputType.FATHER_FIRST_NAME);
                    nameCharAmount = "**";
                }else if (fatherFirstInp.length == 1) {
                    this.addErrorIsTooShort(inputType.FATHER_FIRST_NAME);
                    nameCharAmount = "**";
                } else {
                    if (nameCharAmount !== "**") {
                         nameCharAmount += fatherFirstInp.length;
                         nameCharAmount += "";
                         if (nameCharAmount.length < 2) nameCharAmount = "0" + nameCharAmount;
                    }
                    fatherBlock = fatherFirstInp[fatherFirstInp.length - 2] + fatherFirstInp[fatherFirstInp.length - 1];
                    console.log(fatherBlock);
                }
            }

            let brithdayInp = this.getInputValue(inputType.BIRTHDAY);
            let brithdayBlock = "**";
            if (brithdayInp === null) {
                this.addErrorFieldNotConnected(inputType.BIRTHDAY);
            } else {
                if (/[^0-9]/.test(brithdayInp)) {
                    this.addErrorCannotContainCharacters(inputType.BIRTHDAY);
                } else {
                    brithdayInp = parseInt(brithdayInp);
                    if (isNaN(brithdayInp)) {
                        this.addErrorMustBeANumber(inputType.BIRTHDAY);
                    } else if (brithdayInp <= 0 || brithdayInp >= 32) {
                        this.addErrorMustBeInRange(inputType.BIRTHDAY, 1, 31);
                    } else {
                        if (brithdayInp < 10) {
                            brithdayBlock = "0" + brithdayInp;
                        } else {
                            brithdayBlock = brithdayInp + "";
                        }
                    }
                }
            }
            
            let siblingInp = this.getInputValue(inputType.OLDER_SIBLINGS);
            let siblingBlock = "*";
            if (siblingInp === null) {
                this.addErrorFieldNotConnected(inputType.OLDER_SIBLINGS);
            } else {
                if (/[^0-9]/.test(siblingInp)) {
                    this.addErrorCannotContainCharacters(inputType.OLDER_SIBLINGS);
                } else {
                    siblingInp = parseInt(siblingInp);
                    if (isNaN(siblingInp)) {
                        this.addErrorMustBeANumber(inputType.OLDER_SIBLINGS);
                    } else if (siblingInp < 0 || siblingInp > 9) {
                        this.addErrorMustBeInRange(type, 0, 9);
                    } else {
                        siblingBlock = siblingInp + "";
                    }
                }
            }

            if (fatalError)
                return "***********";

            return (motherBlock + nameCharAmount + fatherBlock + brithdayBlock + motherSecondChar + fatherLastSecondChar + siblingBlock).toUpperCase();
        },
        
        clearErrorList: function() {
            let list = document.querySelector("#errorList");
            let child = list.lastElementChild; 
            while (child) {
                list.removeChild(child);
                child = list.lastElementChild;
            }
        },

        localisedInputTypeFieldName : function(type) {
            switch(type) {
                case inputType.BIRTHDAY:
                    return this.german ? "Geburtstag" : "Brithday";
                case inputType.FATHER_FIRST_NAME:
                    return this.german ? "Vorname des Vaters" : "Father's Firstname";
                case inputType.FATHER_NAME:
                    return this.german ? "Nachname des Vaters" : "Father's Maiden Name";
                case inputType.MOTHER_FIRST_NAME:
                    return this.german ? "Vorname der Mutter" : "Mother's Firstname";
                case inputType.MOTHER_NAME:
                    return this.german ? "Nachname der Mutter" : "Mother's Maiden Name";
                case inputType.OLDER_SIBLINGS:
                    return this.german ? "Anzahl älterer Geschwister" : "Number of older Siblings";
                
                default:
                    return "NOT A TYPE - ERROR [Anker#1442]";
            }
        },

        addErrorFieldNotConnected: function(type){
            this.addError(this.german ? 
                "Inputfeld: " + this.localisedInputTypeFieldName(type) + " nicht verbunden" :
                "Inputfield: " + this.localisedInputTypeFieldName(type) + " not connected");
        },

        addErrorHasToBeAWord : function(type) {
            this.addError(this.german ?
                this.localisedInputTypeFieldName(type) + " muss ein Wort sein" : 
                this.localisedInputTypeFieldName(type) + " has to be a Word");
        },
        
        addErrorMustOnlyContainCharacters : function(type) {
            this.addError(this.german ? 
                this.localisedInputTypeFieldName(type) + " darf nur Buchstaben (keine Umlaute!) enthalten" :
                this.localisedInputTypeFieldName(type) + " must only contain Characters (umlauts are not allowed!)");
        },

        addErrorIsEmpty : function(type) {
            this.addError(this.german ?
                this.localisedInputTypeFieldName(type) + " muss ausgefüllt werden" : 
                this.localisedInputTypeFieldName(type) + " is empty");
        },

        addErrorIsTooShort : function(type) {
            this.addError(this.german ? 
                this.localisedInputTypeFieldName(type) + " ist zu kurz" : 
                this.localisedInputTypeFieldName(type) + " is too short");
        },

        addErrorMustBeANumber : function(type) {
            this.addError(this.german ?
                this.localisedInputTypeFieldName(type) + " muss eine Zahl sein" : 
                this.localisedInputTypeFieldName(type) + " has to be a Number");
        },
        addErrorCannotContainCharacters : function(type) {
            this.addError(this.german ?
                this.localisedInputTypeFieldName(type) + " darf nur Ziffern enthalten" : 
                this.localisedInputTypeFieldName(type) + " must only contain digits");
        },

        addErrorMustBeInRange : function(type, min, max) {
            this.addError(this.german ? 
                this.localisedInputTypeFieldName(type) + " muss zwischen " + min + " und " + max + " liegen" :
                this.localisedInputTypeFieldName(type) + " has to be inbetween " + min +" and " + max);
        },

        addError: function (error) {
            let errorList = document.querySelector("#errorList");
            let newError = document.createElement("li");

            newError.textContent = error;
            errorList.appendChild(newError);
        },

        getInputValue: function(type) {
            if (typeof this.inputs === "undefined") return null;

            for(let inp of this.inputs) {
                if (inp.type === type)
                    return inp.value;
            }
            return null;
        },
        switchLang : function() {
            this.german = !this.german;
        },
        switchLangString : function() {
            return this.german ? "Englisch" : "Deutsch";
        },

        copyCode: function(){
            if (this.getCode().indexOf("*") != -1) {
                alert(this.german ? "Subject Code ist noch Fehlerhaft" : "Subject code contains errors");
            }

            navigator.clipboard.writeText(this.getCode());
        }
    }
});

